{{
        RIC_DS1620
        DS1620 Thermostat Chip control for propeller chip with:
                1x DS1620 for thermostat control
}}

CON

  CMD_WRITE_TH                  = $01
  CMD_WRITE_TL                  = $02
  CMD_WRITE_CONFIG              = $0C
  CMD_STOP_CONVERT              = $22                        
  CMD_READ_COUNTER              = $A0                        
  CMD_READ_TH                   = $A1
  CMD_READ_TL                   = $A2
  CMD_READ_TEMP                 = $AA
  CMD_READ_SLOPE                = $A9
  CMD_READ_CONFIG               = $AC
  CMD_START_CONVERT             = $EE


VAR
  long                PIN_DATA              
  long                PIN_CLOCK
  long                PIN_RESET
  long                lTemperature

  
OBJ
  IO            : "ShiftIO"
  TIME          : "Timing"
  DEBUG         : "FullDuplexSerial"
  SEGMENTLED    : "RIC_7SegmentLED"


PUB Initialize (pinData, pinClock, pinReset)
{{
    Sets local variables and initializes the DS1620 chip
}}

  PIN_DATA := pinData
  PIN_CLOCK := pinClock
  PIN_RESET := pinReset

  InitDS1620


PRI InitDS1620
{{
    Initializes the DS1620 chip
}}

  dira [PIN_DATA]~~
  dira [PIN_CLOCK]~~
  dira [PIN_RESET]~~
  'SET CONFIGURATION TO CPU / FREE-RUN
  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, IO#LsbFirst, CMD_WRITE_CONFIG, 8)
  IO.shiftout (PIN_DATA, PIN_CLOCK, IO#LsbFirst, %10, 8)
  outa[PIN_RESET]~
  TIME.pause1ms (20)
  outa[PIN_CLOCK]~

{
PRI GetTemperatureF_OLD | lRdTemp, lSftTemp, lFTemp

  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, io#LsbFirst, CMD_READ_TEMP, 8)
  lRdTemp := IO.shiftin (PIN_DATA, PIN_CLOCK, io#LsbPre, 9) 
  outa[PIN_RESET]~
  outa[PIN_CLOCK]~ 
  'Dump the .5 bit
  lSftTemp := lRdTemp >> 1
  'Convert to F
  lFTemp := ((lSftTemp * 9) / 5) + 32
  return lFTemp
}   

PUB GetTemperatureF | lRdTemp, lCountRemain, lCountPerC, lTemp, lFTemp
{{
    Gets the temperature in degrees fahrenheit
}}

  'READ TEMP
  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, io#LsbFirst, CMD_READ_TEMP, 8)
  lRdTemp := IO.shiftin (PIN_DATA, PIN_CLOCK, io#LsbPre, 9) 
  outa[PIN_RESET]~
  lRdTemp := lRdTemp >> 1 

  'READ COUNTER
  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, IO#LsbFirst, CMD_READ_COUNTER, 8)
  lCountRemain := IO.shiftin (PIN_DATA, PIN_CLOCK, IO#LsbPre, 9)
  outa[PIN_RESET]~

  'READ SLOPE  
  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, IO#LsbFirst, CMD_READ_SLOPE, 8)
  lCountPerC := IO.shiftin (PIN_DATA, PIN_CLOCK, IO#LsbPre, 9)
  outa[PIN_RESET]~

  'Convert the temp to a readable celcius value
  lTemp := ((lRdTemp * 100) - 25) + (((lCountPerC * 100) - (lCountRemain * 100)) / (lCountPerC * 100))
  if (lTemp // 100) > 50
    lTemp := (lTemp / 100) + 1
  else
    lTemp := (lTemp / 100)

  'Convert the temp to Fahrenheit
  lFTemp := (((lTemp * 100) * 900) / 500) + 3200
  if (lFTemp // 100) > 50
    lFTemp := (lFTemp / 100) + 1
  else
    lFTemp := (lFTemp / 100)

  return lFTemp

    
PUB StartTemperatureConversion
{{
    Sets the DS1620 chip to start temperature conversion
    (i.e. start reading the temp on the chip)
}}

  outa [PIN_CLOCK]~
  outa [PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, io#LsbFirst, CMD_START_CONVERT, 8)
  outa [PIN_RESET]~
  outa [PIN_CLOCK]~
  TIME.pause1ms (20)


PUB SetLowTemperatureF (lTempF) | lTempC, lSftTemp
{{
    Sets the low-temperature threshold for the chip
    (i.e. the temperature below which the "heat" pin goes high)
}}

  'Convert the temp F to temp C
  lTempC := ((lTempF - 32) * 5) / 9

  'Shift that temp 1 to the left (for the decimal place which is ignored)
  lSftTemp := lTempC << 1

  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, IO#LsbFirst, CMD_WRITE_TL, 8)
  IO.shiftout (PIN_DATA, PIN_CLOCK, IO#LsbFirst, lSftTemp, 9)
  outa[PIN_RESET]~
  TIME.pause1ms (20)


PUB GetLowTemperatureF | lRdTemp, lSftTemp, lFTemp
{{
    Gets the current low-temperature threshold
    (i.e. the temperature below which the "heat" pin goes high)
}}

  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, io#LsbFirst, CMD_READ_TL, 8)
  lRdTemp := IO.shiftin (PIN_DATA, PIN_CLOCK, io#LsbPre, 9) 
  outa[PIN_RESET]~
  outa[PIN_CLOCK]~ 
  'Dump the .5 bit
  lSftTemp := lRdTemp >> 1
  'Convert to F
  lFTemp := ((lSftTemp * 9) / 5) + 32
  return lFTemp


PUB SetHighTemperatureF (lTempF) | lTempC, lSftTemp
{{
    Sets the current high-temperature threshold
    (i.e. the temperature above which the "cool" pin goes high)
}}

  'Convert the temp F to temp C
  lTempC := ((lTempF - 32) * 5) / 9

  'Shift that temp 1 to the left (for the decimal place which is ignored)
  lSftTemp := lTempC << 1

  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, IO#LsbFirst, CMD_WRITE_TH, 8)
  IO.shiftout (PIN_DATA, PIN_CLOCK, IO#LsbFirst, lSftTemp, 9)
  outa[PIN_RESET]~
  TIME.pause1ms (20)


PUB GetHighTemperatureF | lRdTemp, lSftTemp, lFTemp
{{
    Gets the current high-temperature threshold
    (i.e. the temperature above which the "cool" pin goes high)
}}

  outa[PIN_CLOCK]~
  outa[PIN_RESET]~~
  IO.shiftout (PIN_DATA, PIN_CLOCK, io#LsbFirst, CMD_READ_TH, 8)
  lRdTemp := IO.shiftin (PIN_DATA, PIN_CLOCK, io#LsbPre, 9) 
  outa[PIN_RESET]~
  outa[PIN_CLOCK]~ 
  'Dump the .5 bit
  lSftTemp := lRdTemp >> 1
  'Convert to F
  lFTemp := ((lSftTemp * 9) / 5) + 32
  return lFTemp
  